package com.moloong.bleach.player.manager;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.game.util.SessionUtil;
import com.moloong.bleach.achievement.structs.ConditionType;
import com.moloong.bleach.activity.activitys.onereward.ConsumeActivity;
import com.moloong.bleach.activity.activitys.onereward.HealthUseActivity;
import com.moloong.bleach.activity.activitys.onereward.SpiritUseActivity;
import com.moloong.bleach.activity.activitys.rank.RankActivity;
import com.moloong.bleach.attribute.structs.AttributeEnum;
import com.moloong.bleach.character.structs.Character;
import com.moloong.bleach.dailymission.structs.MissionID;
import com.moloong.bleach.data.bean.GlobalSettingBean;
import com.moloong.bleach.data.bean.GlobalStringBean;
import com.moloong.bleach.data.bean.PlayerLevelBean;
import com.moloong.bleach.data.bean.SoulAbilitySettingBean;
import com.moloong.bleach.data.manager.DataManager;
import com.moloong.bleach.db.bean.PlayerBean;
import com.moloong.bleach.db.bean.Role;
import com.moloong.bleach.db.bean.RoleState;
import com.moloong.bleach.event.events.impl.LevelUpEvent;
import com.moloong.bleach.event.events.impl.MinusHealthEvent;
import com.moloong.bleach.friend.structs.Type;
import com.moloong.bleach.log.core.LogReason;
import com.moloong.bleach.log.structs.gmcmd.GmQuestionLog;
import com.moloong.bleach.log.structs.hack.PlayerHackRecordLog;
import com.moloong.bleach.log.structs.login.LogoutLog;
import com.moloong.bleach.log.structs.player.PlayerChangeNameLog;
import com.moloong.bleach.log.structs.player.PlayerFightTypeLog;
import com.moloong.bleach.log.structs.player.PlayerLevelUpLog;
import com.moloong.bleach.log.structs.resource.ArenaPointLog;
import com.moloong.bleach.log.structs.resource.ContributionLog;
import com.moloong.bleach.log.structs.resource.GoldLog;
import com.moloong.bleach.log.structs.resource.HealthLog;
import com.moloong.bleach.log.structs.resource.MillionScoreLog;
import com.moloong.bleach.log.structs.resource.RankPointLog;
import com.moloong.bleach.log.structs.resource.SoulLog;
import com.moloong.bleach.log.structs.resource.TrialPointLog;
import com.moloong.bleach.log.structs.worldBoss.WorldBossPointChangeLog;
import com.moloong.bleach.mail.manager.MailManager;
import com.moloong.bleach.mail.structs.MailAttachData;
import com.moloong.bleach.mail.structs.MailType;
import com.moloong.bleach.manager.GameDaoManager;
import com.moloong.bleach.manager.ManagerPool;
import com.moloong.bleach.name.NameManager;
import com.moloong.bleach.name.NameType;
import com.moloong.bleach.player.message.ReqPlayerSettingMessage;
import com.moloong.bleach.player.message.ResChangePlayerNameMessage;
import com.moloong.bleach.player.message.ResGmSuggestToPlayerMessage;
import com.moloong.bleach.player.message.ResPlayerAttrChangeMessage;
import com.moloong.bleach.player.message.ResPlayerHealthRecoverMessage;
import com.moloong.bleach.player.message.ResPlayerSettingMessage;
import com.moloong.bleach.player.message.ResPlayerSkillPointRecoverMessage;
import com.moloong.bleach.player.structs.Player;
import com.moloong.bleach.player.structs.PlayerSetting;
import com.moloong.bleach.robotdata.manager.RobotDataManager;
import com.moloong.bleach.schedule.job.GangBattleApplyJob;
import com.moloong.bleach.schedule.structs.JobRunTime;
import com.moloong.bleach.server.GameServer;
import com.moloong.bleach.server.Global;
import com.moloong.bleach.utils.LogMgr;
import com.moloong.bleach.utils.MessageUtil;
import com.moloong.bleach.utils.RandomUtils;
import com.moloong.bleach.utils.ZipUtil;
import com.moloong.bleach.worldBoss.structs.WorldBossPlayerData;
import com.moloong.game.log.service.GameLogService;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.apache.mina.core.session.IoSession;

/* loaded from: PlayerManager.class */
public class PlayerManager {
    private ConcurrentHashMap<Long, Player> activeAccounts = new ConcurrentHashMap<>();
    private ConcurrentHashMap<Long, Player> activePlayers = new ConcurrentHashMap<>();
    private ConcurrentHashMap<Long, Player> idOnlines = new ConcurrentHashMap<>();
    private ConcurrentHashMap<String, Player> nameOnlines = new ConcurrentHashMap<>();
    private ConcurrentHashMap<Integer, ConcurrentHashMap<Long, Player>> syncPlayerMap = new ConcurrentHashMap<>();
    private AtomicInteger threadId = new AtomicInteger(0);
    private ExecutorService saveRoleStateService = Executors.newSingleThreadExecutor();
    private static Logger log = Logger.getLogger(PlayerManager.class);
    private static final ConcurrentHashMap<Long, Long> hackerPlayers = new ConcurrentHashMap<>();

    public void loadAllPlayers() {
        long curTime = System.currentTimeMillis();
        int parseCount = 0;
        HashSet<String> names = new HashSet<>();
        NameManager.initExistedNames(NameType.PLAYER, names);
        names.addAll(GameDaoManager.getInstance().playerDao.selectAllNames());
        if (!ManagerPool.getInstance().arenaRankingManager.isArenaRankingInit()) {
            for (PlayerBean next : GameDaoManager.getInstance().playerDao.selectall()) {
                try {
                    Player player = (Player) JSON.parseObject(ZipUtil.uncompress(next.getData()), Player.class);
                    ManagerPool.getInstance().arenaRankingManager.init(player);
                    parseCount++;
                } catch (Exception e) {
                    log.error(e, e);
                }
            }
            LogMgr.error(log, (Player) null, String.format("玩家解析完成, 解析数=%d, 耗时:%d秒", Integer.valueOf(parseCount), Long.valueOf((System.currentTimeMillis() - curTime) / 1000)));
        }
    }

    public void question(Player player, String msg) {
        if (StringUtils.isNotBlank(msg)) {
            GameLogService.getInstance().execute(new GmQuestionLog(player, msg));
        }
    }

    public void recoverSkillPoint(Player player) {
        int recoverMaxSkillpoint = ManagerPool.getInstance().soulAbilityManager.getRecoverMaxSkillpoint();
        if (player.getSkillPoint() >= recoverMaxSkillpoint) {
            return;
        }
        SoulAbilitySettingBean config = DataManager.getInstance().soulabilitysettingContainer.getConfig();
        int skillpointAutoRecover = config.getSkillpointAutoRecover();
        long currentTimeMillis = System.currentTimeMillis();
        long lastRecoverTime = player.getSoulAbilityData().getLastRecoverTime();
        long deltaTime = (currentTimeMillis - lastRecoverTime) / 1000;
        if (deltaTime < skillpointAutoRecover) {
            skillpointAutoRecover = (int) (skillpointAutoRecover - deltaTime);
        } else {
            addSkillPoint(player, (int) (deltaTime / skillpointAutoRecover));
            player.getSoulAbilityData().setLastRecoverTime(currentTimeMillis);
            int tmp = (int) (deltaTime % skillpointAutoRecover);
            if (tmp > 0) {
                player.getSoulAbilityData().setLastRecoverTime(currentTimeMillis - (tmp * 1000));
                skillpointAutoRecover -= tmp;
            }
            if (player.getSkillPoint() >= recoverMaxSkillpoint) {
                player.setSkillPoint(recoverMaxSkillpoint);
                skillpointAutoRecover = 0;
            }
        }
        sendSkillPointRecoverMsg(player, skillpointAutoRecover);
        LogMgr.info(log, player, String.format("技能点回复，点数=%d, 时差=%d秒", Integer.valueOf(player.getSkillPoint()), Long.valueOf(deltaTime)));
    }

    public void registerPlayer(Player player) {
        this.activeAccounts.put(Long.valueOf(player.getAccountID()), player);
        this.activePlayers.put(Long.valueOf(player.getPlayerID()), player);
        this.idOnlines.put(Long.valueOf(player.getPlayerID()), player);
        this.nameOnlines.put(NameManager.getName(player.getServerID(), player.getPlayerName()), player);
        addSyncPlayer(player);
        player.setThreadId(getThreadId());
    }

    public void removePlayer(Player player) {
        this.idOnlines.remove(Long.valueOf(player.getPlayerID()));
        this.nameOnlines.remove(NameManager.getName(player.getServerID(), player.getPlayerName()));
    }

    public int getOnlinePlayerCount() {
        return this.idOnlines.size();
    }

    public boolean isPlayerOnline(long playerId) {
        return this.idOnlines.containsKey(Long.valueOf(playerId));
    }

    public ConcurrentHashMap<Long, Player> getOnlinePlayers() {
        return this.idOnlines;
    }

    public List<Player> getOnlinePlayerList() {
        return new ArrayList(this.idOnlines.values());
    }

    public final Player getPlayer(long playerId) {
        return this.idOnlines.get(Long.valueOf(playerId));
    }

    public final Player getPlayer(String playerName) {
        return this.nameOnlines.get(playerName);
    }

    public final Player getPlayerInActive(long accountId) {
        return this.activeAccounts.get(Long.valueOf(accountId));
    }

    public final Player getActivePlayer(long playerId) {
        return this.activePlayers.get(Long.valueOf(playerId));
    }

    public List<Player> getActivePlayerList() {
        return new ArrayList(this.activePlayers.values());
    }

    public void quit(Player player) {
        long curTime = System.currentTimeMillis();
        player.setLogOffTime(curTime);
        ManagerPool.getInstance().gmManager.removeTempGM(player.getPlayerID());
        ManagerPool.getInstance().gangManager.onPlayerOffline(player, player.getLogOffTime());
        ManagerPool.getInstance().buffManager.calcBuffTime(player, false);
        updatePlayer(player);
        ManagerPool.getInstance().limitArenaMgr.sendCharactersToPublicServer(player);
        if (ManagerPool.getInstance().globalPlayerManager.getGlobalPlayer(player.getPlayerID()).getOfflineTime() <= 0) {
            ManagerPool.getInstance().globalPlayerManager.getGlobalPlayer(player.getPlayerID()).setOfflineTime(player.getLogOffTime());
        }
        ManagerPool.getInstance().friendManager.onPlayerValueChange(player, Type.OFFLINE_TIME, 0);
        GameLogService.getInstance().execute(new LogoutLog(player, curTime - player.getLoginTime()));
        ManagerPool.getInstance().loginManager.sendPlayerInfoToInterface(player);
        LogMgr.info(log, player, "退出完成");
    }

    public void updatePlayer(Player player) {
        GameServer.getInstance().getSavePlayerServer().addRole(initRole(player));
        roleStateLog(player, false);
    }

    public void savePlayerImmediately(Player player) {
        GameDaoManager.getInstance().roleDao.update(initRole(player));
    }

    public void insertPlayer(Player player) {
        GameServer.getInstance().getSavePlayerServer().addRole(initRole(player));
        roleStateLog(player, true);
    }

    public void stopServerSavePlayer() {
        int count = 0;
        long curTime = System.currentTimeMillis();
        LogMgr.error(log, (Player) null, String.format("服务器关闭, 保存玩家数据开始(%d)...", Integer.valueOf(this.idOnlines.size())));
        for (Player player : this.idOnlines.values()) {
            if (player != null) {
                try {
                    player.setLogOffTime(curTime);
                    ManagerPool.getInstance().gangManager.onPlayerOffline(player, player.getLogOffTime());
                    ManagerPool.getInstance().buffManager.calcBuffTime(player, false);
                    GameDaoManager.getInstance().roleDao.update(initRole(player));
                    GameLogService.getInstance().execute(new LogoutLog(player, curTime - player.getLoginTime()));
                    count++;
                } catch (Exception e) {
                    log.error(e, e);
                }
            }
        }
        LogMgr.error(log, (Player) null, String.format("服务器关闭, 保存玩家数据结束(%d)...", Integer.valueOf(count)));
    }

    public boolean addExp(Player player, int exp, LogReason reason) {
        int maxLevel;
        int oldLevel;
        boolean levelUp;
        int needExp;
        if (exp <= 0 || (oldLevel = player.getLevel()) > (maxLevel = DataManager.getInstance().globalsettingContainer.getConfig().getMaxPlayerLevel())) {
            return false;
        }
        int totalExp = player.getExp() + exp;
        int nextLevel = player.getLevel() + 1;
        int loop = 0;
        int addedHealth = 0;
        boolean z = false;
        while (true) {
            levelUp = z;
            int i = loop;
            loop++;
            if (i <= maxLevel) {
                if (nextLevel > maxLevel) {
                    totalExp = 0;
                    break;
                }
                PlayerLevelBean levelBean = (PlayerLevelBean) DataManager.getInstance().playerlevelContainer.getMap().get(Integer.valueOf(nextLevel - 1));
                if (levelBean == null || totalExp < (needExp = levelBean.getExp())) {
                    break;
                }
                addedHealth += ((PlayerLevelBean) DataManager.getInstance().playerlevelContainer.getMap().get(Integer.valueOf(nextLevel - 1))).getAddEnergy();
                nextLevel++;
                totalExp -= needExp;
                z = true;
            } else {
                break;
            }
        }
        player.setExp(totalExp);
        if (levelUp) {
            player.setLevel(nextLevel - 1);
            if (addedHealth > 0) {
                addHealth(player, addedHealth, LogReason.ADD_EXP_AND_PLAYER_LEVELUP);
            }
            onPlayerLevelUp(player);
            sendPlayerAttribute(player, 0, player.getLevel());
            updateSyncPlayer(player, oldLevel);
            ManagerPool instance = ManagerPool.getInstance();
            instance.soulPressManager.sendSoulPressInfo(player);
            instance.escortManager.onLogin(player);
            instance.escortManager.changeTeamList(player, 0);
            instance.escortManager.refreshHealth(player);
            instance.soulSkeletonManager.sendSoulSkeletonInfo(player);
            instance.achievementManager.updateAchievement(player, ConditionType.Condition1, true);
            instance.globalPlayerManager.OnPlayerLevelUp(player);
            instance.destinyManager.onPlayerLevelUp(player);
            instance.millionsFortuneManager.onPlayerLevelUp(player);
            instance.foodManager.onPlayerLevelUp(player);
            instance.dailyMissionMgr.activeDailyMission(player);
            instance.soulTrialMgr.activeSoulTrial(player);
            instance.shopManager.onPlayerLevelUp(player);
            instance.specialShopManager.openedFunction(player);
            instance.specialTrainingManager.onPlayerLevelUp(player);
            instance.limitArenaMgr.onPlayerLevelUp(player);
            instance.rankingMgr.onPlayerLevelUp(player, oldLevel);
            instance.springManager.onLevelUp(player, oldLevel);
            instance.activityMgr.update(RankActivity.class, player);
            instance.soulResonanceMgr.onLevelUp(player, oldLevel);
            instance.playerLevelRankingMgr.addRankingData(player, false);
            instance.levelRankingActivityManager.addRankingData(player);
            if (instance.memoryRoadManager.openFunction(player)) {
                instance.memoryRoadManager.sendMemoryRoadInfos(player, true);
            }
            if (instance.advanceTrialManager.openFunction(player)) {
                instance.advanceTrialManager.sendAdvanceTrialInfos(player, true, true, true, true);
            }
            instance.unlimitFightManager.sendUnlimitFightInfos(player);
            instance.topArenaMgr.active(player, false);
            instance.topArenaMgr.sendDataToPublicServerOnPlayerLevelUp(player);
            instance.topArenaMgr.sendRankingData(player);
            instance.globalPlayerManager.getGlobalPlayer(player.getPlayerID()).setPlayerLevel(nextLevel - 1);
            instance.friendManager.onPlayerValueChange(player, Type.PLAYER_LEVEL, nextLevel - 1);
            ManagerPool.getInstance().activityMgr.sendAllActivityMessage(player);
            instance.retrieveManager.onLevelUp(player, oldLevel);
            instance.worldBossManager.refreshCharacterHp(player);
            instance.worldBossManager.sendWorldBossSyncMessage(player);
            instance.peerlessManager.active(player);
            instance.peerlessManager.sendPeerlessBaseInfo(player);
            instance.activityExtendManager.sendAllActivityMessage(player);
            instance.redPacketManager.updateData(player);
            instance.staticBloodManager.refreshPlayerStaticHp(player);
            instance.ichigoTrialManager.activeFunction(player);
            instance.collegeManager.activeCollege(player);
            instance.eventManager.publishEvent(new LevelUpEvent(player, true, player.getLevel()));
            instance.petBoniManager.activeFunction(player);
            instance.exploreManager.levelUpActiveExplore(player);
            instance.hougyokuPowerManager.sendHougyokuPowerSettingMessage(player);
            ManagerPool.getInstance().attributeManager.calcPlayerAttribute(AttributeEnum.HOUGYOKUPOWER, player, true);
            PlayerLevelUpLog playerLevelUpLog = new PlayerLevelUpLog(player, oldLevel, player.getLevel(), LogReason.ADD_EXP_AND_PLAYER_LEVELUP);
            GameLogService.getInstance().execute(playerLevelUpLog);
        }
        sendPlayerAttribute(player, 1, totalExp);
        return true;
    }

    public boolean addGold(Player player, int gold, LogReason reason) {
        if (gold <= 0 || player.getGold() >= 2000000000) {
            return false;
        }
        int oldGold = player.getGold();
        int totalGold = Math.min(oldGold + gold, 2000000000);
        player.setGold(totalGold);
        sendPlayerAttribute(player, 2, totalGold);
        GoldLog goldLog = new GoldLog(player, oldGold, totalGold, reason);
        GameLogService.getInstance().execute(goldLog);
        return true;
    }

    public boolean minusGold(Player player, int gold, LogReason reason) {
        int oldGold;
        int curGold;
        if (gold <= 0 || (curGold = (oldGold = player.getGold()) - gold) < 0) {
            return false;
        }
        player.setGold(curGold);
        sendPlayerAttribute(player, 2, curGold);
        ManagerPool.getInstance().activityMgr.updateEx(SpiritUseActivity.class, player, gold, false);
        GoldLog goldLog = new GoldLog(player, oldGold, curGold, reason);
        GameLogService.getInstance().execute(goldLog);
        return true;
    }

    public boolean addSoul(Player player, int soul, LogReason reason) {
        int oldSoul;
        if (soul <= 0 || (oldSoul = player.getSoul()) >= 2000000000) {
            return false;
        }
        int totalSoul = Math.min(oldSoul + soul, 2000000000);
        player.setSoul(totalSoul);
        sendPlayerAttribute(player, 4, totalSoul);
        SoulLog soulLog = new SoulLog(player, oldSoul, totalSoul, reason);
        GameLogService.getInstance().execute(soulLog);
        return true;
    }

    public boolean minusSoul(Player player, int soul, LogReason reason) {
        int oldSoul;
        int curSoul;
        if (soul <= 0 || (curSoul = (oldSoul = player.getSoul()) - soul) < 0) {
            return false;
        }
        player.setSoul(curSoul);
        sendPlayerAttribute(player, 4, curSoul);
        ManagerPool.getInstance().activityMgr.updateEx(ConsumeActivity.class, player, soul, false);
        SoulLog soulLog = new SoulLog(player, oldSoul, curSoul, reason);
        GameLogService.getInstance().execute(soulLog);
        ManagerPool.getInstance().consumeRankingActivityManager.addRankingData(player, soul);
        ManagerPool.getInstance().rechargeRankManager.uploadRechargeConsumeData(player, soul, 3);
        ManagerPool.getInstance().rechargeRankManager.uploadRechargeConsumeData(player, soul, 4);
        return true;
    }

    public void refreshHealth(Player player) {
        PlayerLevelBean playerLevelBean = (PlayerLevelBean) DataManager.getInstance().playerlevelContainer.getMap().get(Integer.valueOf(player.getLevel()));
        if (playerLevelBean == null) {
            sendHealthRecover(player, 0);
            return;
        }
        int maxEnergy = playerLevelBean.getMaxEnergy();
        int oldHealth = player.getHealth();
        if (oldHealth >= maxEnergy) {
            sendHealthRecover(player, 0);
            return;
        }
        long curTime = System.currentTimeMillis();
        long deltaTime = (curTime - player.getHealthRecoverTime()) / 1000;
        if (deltaTime <= 0) {
            return;
        }
        int energyAutoRecover = DataManager.getInstance().globalsettingContainer.getConfig().getEnergyAutoRecover();
        long health = deltaTime / energyAutoRecover;
        long nextTime = energyAutoRecover - (deltaTime % energyAutoRecover);
        if (nextTime < 0) {
            nextTime = 0;
        }
        if (health > 0) {
            player.setHealth(player.getHealth() + ((int) health));
            player.setHealthRecoverTime(curTime - (nextTime * 1000));
        }
        if (player.getHealth() > maxEnergy) {
            player.setHealth(maxEnergy);
            player.setHealthRecoverTime(curTime);
            sendHealthRecover(player, 0);
        } else {
            sendHealthRecover(player, (int) nextTime);
            LogMgr.info(log, player, String.format("上线更新体力, cd=%d, passtime=%d, old health=%d, add health=%d, new health=%d", Integer.valueOf(energyAutoRecover), Long.valueOf(deltaTime), Integer.valueOf(oldHealth), Long.valueOf(health), Integer.valueOf(player.getHealth())));
        }
    }

    public void sendGmSuggest(Player player) {
        GlobalSettingBean globalSettingBean = (GlobalSettingBean) DataManager.getInstance().globalsettingContainer.getList().get(0);
        if (globalSettingBean == null) {
            return;
        }
        ResGmSuggestToPlayerMessage message = new ResGmSuggestToPlayerMessage();
        message.setContext(globalSettingBean.getContactUsText());
        MessageUtil.tellPlayer(player, message);
    }

    public void resetGangBattleData(Player player) {
        ManagerPool.getInstance().gangManager.deleteGangBattlePlayerInfo(player);
        JobRunTime jobRumTime = ManagerPool.getInstance().jobRunTimeManager.getJobRunTime(GangBattleApplyJob.class);
        if (null == jobRumTime || player.getGangBattleInfo().getResetDataTimeMs() < jobRumTime.getLastRunTime()) {
            player.getGangBattleInfo().setResetDataTimeMs(System.currentTimeMillis());
            ManagerPool.getInstance().gangManager.removeGangBattleApply(player);
            player.getGangBattleInfo().setResetDataTimeMs(System.currentTimeMillis());
            player.getGangBattleInfo().setApplyed(false);
            player.getGangBattleInfo().setApplyTime(0L);
            player.getGangBattleInfo().setFightTime(0);
            player.getGangBattleInfo().setWinTime(0);
            player.getGangBattleInfo().setOppoGangName("");
            player.getGangBattleInfo().setOppoLeaderName("");
            player.getGangBattleInfo().getFighterCharacterIds().clear();
            player.getGangBattleInfo().getResonanceIds().clear();
            player.getGangBattleInfo().getOppoFighters().clear();
            player.getGangBattleInfo().getFightedPlayerIds().clear();
            LogMgr.info(log, player, "番队战报名开始,清除之前的报名信息");
        }
    }

    public void recoverHealth(Player player) {
        PlayerLevelBean playerLevelBean = (PlayerLevelBean) DataManager.getInstance().playerlevelContainer.getMap().get(Integer.valueOf(player.getLevel()));
        if (playerLevelBean == null) {
            sendHealthRecover(player, 0);
            return;
        }
        int maxEnergy = playerLevelBean.getMaxEnergy();
        if (player.getHealth() >= maxEnergy) {
            sendHealthRecover(player, 0);
            return;
        }
        int oldHealth = player.getHealth();
        int energyAutoRecover = DataManager.getInstance().globalsettingContainer.getConfig().getEnergyAutoRecover();
        long currentTimeMillis = System.currentTimeMillis();
        long healthRecoverTime = player.getHealthRecoverTime();
        long deltaTime = (currentTimeMillis - healthRecoverTime) / 1000;
        int recover = (int) (deltaTime / energyAutoRecover);
        if (deltaTime < energyAutoRecover) {
            energyAutoRecover = (int) (energyAutoRecover - deltaTime);
        } else {
            player.setHealth(player.getHealth() + recover);
            player.setHealthRecoverTime(currentTimeMillis);
            int tmp = (int) (deltaTime % energyAutoRecover);
            if (tmp > 0) {
                player.setHealthRecoverTime(currentTimeMillis - (tmp * 1000));
                energyAutoRecover -= tmp;
            }
        }
        if (player.getHealth() >= maxEnergy) {
            player.setHealth(maxEnergy);
            energyAutoRecover = 0;
        }
        sendHealthRecover(player, energyAutoRecover);
        LogMgr.info(log, player, String.format("在线更新体力, cd=%d, passtime=%d, old health=%d, add health=%d, new health=%d", Integer.valueOf(energyAutoRecover), Long.valueOf(deltaTime), Integer.valueOf(oldHealth), Integer.valueOf(recover), Integer.valueOf(player.getHealth())));
    }

    private void sendHealthRecover(Player player, int cdTime) {
        ResPlayerHealthRecoverMessage message = new ResPlayerHealthRecoverMessage();
        message.setHealth(player.getHealth());
        message.setCdTime(cdTime);
        MessageUtil.tellPlayer(player, message);
    }

    public boolean minusHealth(Player player, int health, LogReason reason) {
        int oldHealth;
        int curHealth;
        if (health <= 0 || (curHealth = (oldHealth = player.getHealth()) - health) < 0) {
            return false;
        }
        int maxEnergy = ((PlayerLevelBean) DataManager.getInstance().playerlevelContainer.getMap().get(Integer.valueOf(player.getLevel()))).getMaxEnergy();
        player.setHealth(curHealth);
        boolean flag = oldHealth >= maxEnergy && curHealth < maxEnergy;
        if (flag) {
            player.setHealthRecoverTime(System.currentTimeMillis());
            sendHealthRecover(player, DataManager.getInstance().globalsettingContainer.getConfig().getEnergyAutoRecover());
        } else {
            sendPlayerAttribute(player, 7, curHealth);
        }
        ManagerPool.getInstance().dailyMissionMgr.updateDailyMissionEx(player, MissionID.Mission21, health);
        ManagerPool.getInstance().gangManager.onMinusHealth(player, oldHealth - player.getHealth());
        ManagerPool.getInstance().activityMgr.updateEx(HealthUseActivity.class, player, oldHealth - player.getHealth(), false);
        ManagerPool.getInstance().eventManager.publishEvent(new MinusHealthEvent(player, true, health));
        HealthLog healthLog = new HealthLog(player, oldHealth, player.getHealth(), reason);
        GameLogService.getInstance().execute(healthLog);
        return true;
    }

    public boolean addHealth(Player player, int health, LogReason reason) {
        int oldHealth;
        if (health <= 0 || (oldHealth = player.getHealth()) >= 999999) {
            return false;
        }
        int totalHealth = Math.min(oldHealth + health, 999999);
        player.setHealth(totalHealth);
        sendPlayerAttribute(player, 7, totalHealth);
        HealthLog healthLog = new HealthLog(player, oldHealth, totalHealth, reason);
        GameLogService.getInstance().execute(healthLog);
        return true;
    }

    public boolean addStar(Player player, int star) {
        if (star <= 0) {
            return false;
        }
        int oldStar = player.getStar();
        int curStar = oldStar + star;
        if (curStar > 10000) {
            curStar = 10000;
            star = 10000 - oldStar;
        }
        player.setStar(curStar);
        player.setTotalStar(player.getTotalStar() + star);
        sendPlayerAttribute(player, 6, curStar);
        return true;
    }

    public boolean minusStar(Player player, int star) {
        int oldStar;
        if (star <= 0 || (oldStar = player.getStar()) < star) {
            return false;
        }
        int curStar = oldStar - star;
        player.setStar(curStar);
        sendPlayerAttribute(player, 6, curStar);
        return true;
    }

    public boolean addArenaPoint(Player player, int point, LogReason reason) {
        if (point < 0) {
            return false;
        }
        int oldPoint = player.getArenaPoint();
        player.setArenaPoint(oldPoint + point);
        sendPlayerAttribute(player, 5, player.getArenaPoint());
        GameLogService.getInstance().execute(new ArenaPointLog(player, oldPoint, player.getArenaPoint(), reason));
        return true;
    }

    public boolean minusArenaPoint(Player player, int point, LogReason reason) {
        int oldPoint;
        if (point < 0 || (oldPoint = player.getArenaPoint()) < point) {
            return false;
        }
        player.setArenaPoint(oldPoint - point);
        sendPlayerAttribute(player, 5, player.getArenaPoint());
        GameLogService.getInstance().execute(new ArenaPointLog(player, oldPoint, player.getArenaPoint(), reason));
        return true;
    }

    public boolean addSkillPoint(Player player, int point) {
        if (point < 0) {
            return false;
        }
        int oldPoint = player.getSkillPoint();
        player.setSkillPoint(oldPoint + point);
        ManagerPool.getInstance().soulAbilityManager.sendSkillPointCount(player);
        return true;
    }

    public void sendSkillPointRecoverMsg(Player player, int cdTime) {
        ResPlayerSkillPointRecoverMessage message = new ResPlayerSkillPointRecoverMessage();
        message.setSkillPoint(player.getSkillPoint());
        message.setCdTime(cdTime);
        MessageUtil.tellPlayer(player, message);
    }

    public boolean minusSkillPoint(Player player, int point, Character character, LogReason reason) {
        int oldPoint;
        int curPoint;
        if (point < 0 || (curPoint = (oldPoint = player.getSkillPoint()) - point) < 0) {
            return false;
        }
        int recoverMaxSkillpoint = ManagerPool.getInstance().soulAbilityManager.getRecoverMaxSkillpoint();
        boolean flag = oldPoint >= recoverMaxSkillpoint && curPoint < recoverMaxSkillpoint;
        if (flag) {
            player.getSoulAbilityData().setLastRecoverTime(System.currentTimeMillis());
        }
        player.setSkillPoint(curPoint);
        ManagerPool.getInstance().soulAbilityManager.sendSkillPointCount(player);
        return true;
    }

    public boolean changeSkillPoint(Player player, int currentPoint) {
        if (currentPoint < 0 || currentPoint > ManagerPool.getInstance().soulAbilityManager.getMaxSkillpoint()) {
            return false;
        }
        int recoverMaxSkillpoint = ManagerPool.getInstance().soulAbilityManager.getRecoverMaxSkillpoint();
        if (player.getSkillPoint() >= recoverMaxSkillpoint && currentPoint < recoverMaxSkillpoint) {
            player.getSoulAbilityData().setLastRecoverTime(System.currentTimeMillis());
        }
        player.setSkillPoint(currentPoint);
        ManagerPool.getInstance().soulAbilityManager.sendSkillPointCount(player);
        return true;
    }

    public boolean addTrialPoint(Player player, int point, LogReason reason) {
        if (point < 0) {
            return false;
        }
        int oldPoint = player.getTrialPoint();
        player.setTrialPoint(oldPoint + point);
        sendPlayerAttribute(player, 9, player.getTrialPoint());
        GameLogService.getInstance().execute(new TrialPointLog(player, oldPoint, player.getTrialPoint(), reason));
        return true;
    }

    public boolean minusTrialPoint(Player player, int point, LogReason reason) {
        int oldPoint;
        if (point < 0 || (oldPoint = player.getTrialPoint()) < point) {
            return false;
        }
        player.setTrialPoint(oldPoint - point);
        sendPlayerAttribute(player, 9, player.getTrialPoint());
        GameLogService.getInstance().execute(new TrialPointLog(player, oldPoint, player.getTrialPoint(), reason));
        return true;
    }

    public boolean addContribution(Player player, int contribution, LogReason reason) {
        if (contribution <= 0) {
            return false;
        }
        int before = player.getGangContribution();
        player.setGangContribution(player.getGangContribution() + contribution);
        sendPlayerAttribute(player, 8, player.getGangContribution());
        ContributionLog.log(player, before, player.getGangContribution(), reason);
        return true;
    }

    public boolean minusContribution(Player player, int contribution, LogReason reason) {
        int oldValue;
        if (contribution <= 0 || (oldValue = player.getGangContribution()) < contribution) {
            return false;
        }
        player.setGangContribution(oldValue - contribution);
        sendPlayerAttribute(player, 8, player.getGangContribution());
        ContributionLog.log(player, oldValue, player.getGangContribution(), reason);
        return true;
    }

    public boolean addButterfly(Player player, int count) {
        if (count <= 0) {
            return false;
        }
        player.setButterflyCount(player.getButterflyCount() + count);
        sendPlayerAttribute(player, 11, player.getButterflyCount());
        return true;
    }

    public boolean minusButterfly(Player player, int count) {
        int oldValue;
        if (count <= 0 || (oldValue = player.getButterflyCount()) < count) {
            return false;
        }
        player.setButterflyCount(oldValue - count);
        sendPlayerAttribute(player, 11, player.getButterflyCount());
        return true;
    }

    public boolean addRankPoint(Player player, int point, LogReason reason) {
        if (point <= 0) {
            return false;
        }
        int oldPoint = player.getRankPoint();
        player.setRankPoint(oldPoint + point);
        sendPlayerAttribute(player, 14, player.getRankPoint());
        RankPointLog rankPointLog = new RankPointLog(player, oldPoint, player.getRankPoint(), reason);
        GameLogService.getInstance().execute(rankPointLog);
        return true;
    }

    public boolean minusRankPoint(Player player, int point, LogReason reason) {
        int oldValue;
        if (point <= 0 || (oldValue = player.getRankPoint()) < point) {
            return false;
        }
        player.setRankPoint(oldValue - point);
        sendPlayerAttribute(player, 14, player.getRankPoint());
        RankPointLog rankPointLog = new RankPointLog(player, oldValue, player.getRankPoint(), reason);
        GameLogService.getInstance().execute(rankPointLog);
        return true;
    }

    public boolean addMillionScore(Player player, int millionScore, LogReason reason) {
        if (millionScore <= 0) {
            return false;
        }
        int oldMillionScore = player.getMillionScore();
        player.setMillionScore(oldMillionScore + millionScore);
        sendPlayerAttribute(player, 15, player.getMillionScore());
        MillionScoreLog millionScoreLog = new MillionScoreLog(player, oldMillionScore, player.getMillionScore(), reason);
        GameLogService.getInstance().execute(millionScoreLog);
        return true;
    }

    public boolean minusMillionScore(Player player, int millionScore, LogReason reason) {
        int oldValue;
        if (millionScore <= 0 || (oldValue = player.getMillionScore()) < millionScore) {
            return false;
        }
        player.setMillionScore(oldValue - millionScore);
        sendPlayerAttribute(player, 15, player.getMillionScore());
        MillionScoreLog millionScoreLog = new MillionScoreLog(player, oldValue, player.getMillionScore(), reason);
        GameLogService.getInstance().execute(millionScoreLog);
        return true;
    }

    private final int getThreadId() {
        int id = this.threadId.getAndIncrement() % Global.threadNum;
        if (id < 0) {
            id += Global.threadNum;
        }
        return id;
    }

    public Role initRole(Player player) {
        Role role = new Role();
        role.setAccountID(player.getAccountID());
        role.setAccountName(player.getAccountName());
        role.setPlayerID(player.getPlayerID());
        role.setPlayerName(player.getPlayerName());
        role.setServerID(player.getServerID());
        role.setVipLevel(player.getVipLevel());
        role.setPlayerLevel(player.getLevel());
        role.setExp(player.getExp());
        role.setCharge(player.getCharge());
        role.setHealth(player.getHealth());
        role.setSoul(player.getSoul());
        role.setGold(player.getGold());
        role.setStar(player.getStar());
        role.setRank(player.getArenaData().getRank());
        role.setButterfly(player.getButterflyCount());
        role.setLogoffTime(player.getLogOffTime());
        try {
            role.setData(ZipUtil.compress(JSON.toJSONString(player, new SerializerFeature[]{SerializerFeature.DisableCircularReferenceDetect})));
        } catch (Exception e) {
            log.error(e, e);
        }
        return role;
    }

    public void onPlayerLevelUp(Player player) {
        for (Character it : player.getCharacters().getCharMap().values()) {
            ManagerPool.getInstance().attributeManager.calcPlayerAttribute(AttributeEnum.BASE, player, it, true);
        }
        ManagerPool.getInstance().gangManager.onPlayerLevelUp(player, player.getLevel());
    }

    public void sendPlayerAttribute(Player player, int id, int value) {
        ResPlayerAttrChangeMessage msg = new ResPlayerAttrChangeMessage();
        msg.setAttrbuteId(id);
        msg.setAttrbuteValue(value);
        MessageUtil.tellPlayer(player, msg);
    }

    public int getFightPower(Player player) {
        return (int) Math.ceil(player.getFightPower());
    }

    public Map<Long, Player> getSyncPlayer(int level) {
        return this.syncPlayerMap.get(Integer.valueOf(level));
    }

    public void updateSyncPlayer(Player player, int oldLevel) {
        removeSyncPlayer(player, oldLevel);
        addSyncPlayer(player);
    }

    private void addSyncPlayer(Player player) {
        this.syncPlayerMap.putIfAbsent(Integer.valueOf(player.getLevel()), new ConcurrentHashMap<>());
        this.syncPlayerMap.get(Integer.valueOf(player.getLevel())).put(Long.valueOf(player.getPlayerID()), player);
    }

    private void removeSyncPlayer(Player player, int level) {
        ConcurrentHashMap<Long, Player> tmpMap = this.syncPlayerMap.get(Integer.valueOf(level));
        if (tmpMap != null) {
            tmpMap.remove(Long.valueOf(player.getPlayerID()));
        }
    }

    public Character getCharacter(Player player, int characterId) {
        return (Character) player.getCharacters().getCharMap().get(Integer.valueOf(characterId));
    }

    public void ActivePlayerRefresh() {
        long curTime = System.currentTimeMillis();
        long activeTime = curTime - 600000;
        for (Player player : this.activePlayers.values()) {
            long playerID = player.getPlayerID();
            if (!this.idOnlines.containsKey(Long.valueOf(playerID)) && player.getLogOffTime() < activeTime) {
                long accountID = player.getAccountID();
                this.activeAccounts.remove(Long.valueOf(accountID));
                this.activePlayers.remove(Long.valueOf(playerID));
                removeSyncPlayer(player, player.getLevel());
                IoSession playerSession = ManagerPool.getInstance().sessionManager.getPlayerSession(playerID);
                ManagerPool.getInstance().sessionManager.removePlayerSession(playerID);
                ManagerPool.getInstance().sessionManager.removeAccountSession(accountID);
                if (playerSession != null) {
                    SessionUtil.closeSession(playerSession, "从活跃列表移除", true);
                }
            }
        }
        long usedTime = System.currentTimeMillis() - curTime;
        log.error(String.format("active player refresh over, left{%d}, time{%d} ms", Integer.valueOf(this.activePlayers.size()), Long.valueOf(usedTime)));
        int freezePlayer = 0;
        long endTime = System.currentTimeMillis() + 259200000;
        Iterator<Map.Entry<Long, Long>> iterex = hackerPlayers.entrySet().iterator();
        while (iterex.hasNext()) {
            Map.Entry<Long, Long> next = iterex.next();
            long playerId = next.getKey().longValue();
            long time = next.getValue().longValue();
            if (curTime >= time) {
                int i = freezePlayer;
                freezePlayer++;
                if (i <= 10) {
                    ManagerPool.getInstance().blacklistManager.freezePlayer(playerId, endTime);
                    log.error("反外挂封禁账号：" + playerId);
                    iterex.remove();
                }
            }
        }
    }

    public int getTopThreeFightPower(Player player) {
        List<Integer> tmpList = new ArrayList<>();
        Collection<Character> values = player.getCharacters().getCharMap().values();
        for (Character value : values) {
            tmpList.add(Integer.valueOf((int) value.getAttributeData().getFightPower()));
        }
        Collections.sort(tmpList, Collections.reverseOrder());
        int power = 0;
        int loop = Math.min(3, tmpList.size());
        for (int i = 0; i < loop; i++) {
            power += tmpList.get(i).intValue();
        }
        return power / loop;
    }

    public int getPlayerNumById(long playerId) {
        int playerNum = this.idOnlines.get(Long.valueOf(playerId)) != null ? 1 : 0;
        if (playerNum == 0) {
            playerNum = GameDaoManager.getInstance().playerDao.selectNumByPlayerId(playerId);
        }
        return playerNum;
    }

    public void playerHackRecord(Player player, int type) {
        int kickCheater2 = DataManager.getInstance().globalsettingContainer.getConfig().getKickCheater2();
        if (kickCheater2 > 0 && type > 2 && !hackerPlayers.contains(Long.valueOf(player.getPlayerID()))) {
            hackerPlayers.put(Long.valueOf(player.getPlayerID()), Long.valueOf(System.currentTimeMillis() + RandomUtils.random(60000L, 600000L)));
        }
        GameLogService.getInstance().execute(new PlayerHackRecordLog(player, type));
        LogMgr.error(log, player, "cheat hack type :" + type);
    }

    public void playerFightTypeRecord(Player player, int type) {
        GameLogService.getInstance().execute(new PlayerFightTypeLog(player, type));
    }

    private void roleStateLog(Player player, boolean isCreate) {
        RoleState roleState = new RoleState(player);
        Date now = new Date();
        if (isCreate) {
            roleState.setCreateTime(now);
        }
        roleState.setUpdateTime(now);
        this.saveRoleStateService.execute(new 1(this, isCreate, roleState));
    }

    public void changeName(Player player, String name, boolean isNeedCard) {
        String logOldName = player.getPlayerName();
        String oldName = NameManager.getName(player.getServerID(), player.getPlayerName());
        int serverID = player.getServerID();
        if (isNeedCard) {
            int checkPlayerName = ManagerPool.getInstance().nameManager.checkPlayerName(name, serverID);
            if (checkPlayerName != 0) {
                sendChangePlayerNameResult(player, checkPlayerName, name);
                return;
            }
            int changeNameItemId = DataManager.getInstance().globalsettingContainer.getConfig().getChangeNameItemId();
            boolean removeItem = ManagerPool.getInstance().itemManager.removeItem(player, changeNameItemId, 1, LogReason.CHANGE_PLAYER_NAME);
            if (!removeItem) {
                return;
            }
        }
        String newName = NameManager.getName(player.getServerID(), name);
        player.setPlayerName(name);
        ManagerPool.getInstance().friendManager.onPlayerNameChange(player);
        ManagerPool.getInstance().globalPlayerManager.setPlayerName(player.getPlayerID(), newName);
        ManagerPool.getInstance().advanceTrialManager.modifyPlayerName(player.getPlayerID());
        ManagerPool.getInstance().ichigoTrialManager.modifyPlayerName(player.getPlayerID(), newName);
        this.nameOnlines.remove(oldName);
        this.nameOnlines.put(newName, player);
        ManagerPool.getInstance().gangManager.changePlayerName(player);
        ManagerPool.getInstance().rankingMgr.updateRankingDataNames(player);
        sendChangePlayerNameResult(player, 0, NameManager.getName(serverID, name));
        GameLogService.getInstance().execute(new PlayerChangeNameLog(player, logOldName, name));
        LogMgr.info(log, player, "玩家改名完成。");
    }

    private void sendChangePlayerNameResult(Player player, int result, String name) {
        ResChangePlayerNameMessage message = new ResChangePlayerNameMessage();
        message.setResult(result);
        message.setName(name);
        MessageUtil.tellPlayer(player, message);
    }

    public void saveOrSendPlayerSetting(Player player, ReqPlayerSettingMessage msg) {
        int operation = msg.getOperation();
        PlayerSetting playerSetting = player.getPlayerSetting();
        if (operation != 0) {
            if (operation == 1) {
                if (playerSetting == null) {
                    playerSetting = new PlayerSetting();
                    playerSetting.setBackgroundVolume(msg.getBackgroundVolume());
                    playerSetting.setControllMode(msg.getControllMode());
                    playerSetting.setEffectVolume(msg.getEffectVolume());
                    playerSetting.setShowNotification(msg.getShowNotification());
                } else {
                    playerSetting.setBackgroundVolume(msg.getBackgroundVolume());
                    playerSetting.setControllMode(msg.getControllMode());
                    playerSetting.setEffectVolume(msg.getEffectVolume());
                    playerSetting.setShowNotification(msg.getShowNotification());
                }
                player.setPlayerSetting(playerSetting);
                return;
            }
            return;
        }
        ResPlayerSettingMessage resPlayerSettingMessage = new ResPlayerSettingMessage();
        if (playerSetting == null) {
            resPlayerSettingMessage.setEffectVolume(10000);
            int combatOperation = DataManager.getInstance().globalsettingContainer.getConfig().getCombatOperation();
            if (combatOperation == 2) {
                combatOperation = 0;
            }
            resPlayerSettingMessage.setControllMode(combatOperation);
            resPlayerSettingMessage.setBackgroundVolume(10000);
            resPlayerSettingMessage.setSowNotification(0);
        } else {
            resPlayerSettingMessage.setEffectVolume(playerSetting.getEffectVolume());
            resPlayerSettingMessage.setControllMode(playerSetting.getControllMode());
            resPlayerSettingMessage.setBackgroundVolume(playerSetting.getBackgroundVolume());
            resPlayerSettingMessage.setSowNotification(playerSetting.getShowNotification());
        }
        MessageUtil.tellPlayer(player, resPlayerSettingMessage);
    }

    public boolean addWorldBossPoint(Player player, int point) {
        if (point <= 0) {
            return false;
        }
        WorldBossPlayerData playerData = player.getWorldBossPlayerData();
        int oldPoint = playerData.getPoint();
        int nowPoint = oldPoint + point;
        playerData.setPoint(nowPoint);
        sendPlayerAttribute(player, 18, nowPoint);
        GameLogService.getInstance().execute(new WorldBossPointChangeLog(player, oldPoint, nowPoint, LogReason.WORLD_BOSS_FIGHT_REWARD));
        return true;
    }

    public boolean minusWorldBossPoint(Player player, int point) {
        WorldBossPlayerData playerData = player.getWorldBossPlayerData();
        int oldPoint = playerData.getPoint();
        if (point > oldPoint) {
            LogMgr.error(log, player, String.format("世界boss积分不够，当前积分{%d},需要的积分{%d}", Integer.valueOf(oldPoint), Integer.valueOf(point)));
            return false;
        }
        int nowPoint = oldPoint - point;
        playerData.setPoint(nowPoint);
        sendPlayerAttribute(player, 18, nowPoint);
        WorldBossPointChangeLog worldBossPointChangeLog = new WorldBossPointChangeLog(player, oldPoint, nowPoint, LogReason.WORLD_BOSS_SHOP_BUY);
        GameLogService.getInstance().execute(worldBossPointChangeLog);
        return true;
    }

    public void handerOldIllegalPlayerName(Player player) {
        try {
            if (!ManagerPool.getInstance().nameManager.isOldNameCheck()) {
                return;
            }
            String currVersion = ManagerPool.getInstance().nameManager.getSensitiveWordsToolsVersion();
            if (StringUtils.isBlank(currVersion) || currVersion.equalsIgnoreCase(player.getNameCheckvVersion())) {
                return;
            }
            if (!ManagerPool.getInstance().nameManager.isLegal(player.getPlayerName())) {
                String randomName = RobotDataManager.getRandomName();
                String oldPlayerName = player.getPlayerName();
                changeName(player, randomName, false);
                sendChangePlayerCardMail(player, oldPlayerName);
                LogMgr.error(log, player, String.format("检测到玩家老名字不合法，强制改名，老名字：%s,新名字：%s", oldPlayerName, randomName));
            }
            player.setNameCheckvVersion(currVersion);
        } catch (Exception e) {
            LogMgr.error(log, player, "检测玩家老名字错误：" + e.getMessage());
        }
    }

    private void sendChangePlayerCardMail(Player player, String oldName) {
        int changeNameItemId = DataManager.getInstance().globalsettingContainer.getConfig().getChangeNameItemId();
        List<MailAttachData> rewardItems = new ArrayList<>();
        MailAttachData mailAttachData = new MailAttachData(changeNameItemId, 1);
        rewardItems.add(mailAttachData);
        GlobalStringBean globalTitle = (GlobalStringBean) DataManager.getInstance().globalstringContainer.getMap().get(3117);
        GlobalStringBean globalContentFormat = (GlobalStringBean) DataManager.getInstance().globalstringContainer.getMap().get(3118);
        if (globalTitle == null || globalContentFormat == null) {
            log.error(String.format("系统强制更名发送玩家{%d}改名卡失败!!!globalstring配置表找不到对应的文本ID{%d}或{%d}", Long.valueOf(player.getPlayerID()), 3117, 3118));
            return;
        }
        String title = globalTitle.getText();
        String contentFormat = globalContentFormat.getText();
        ManagerPool.getInstance().mailManager.sendMail(MailType.OPERATING, 0, player.getPlayerID(), MailManager.getDefaultSender(), title, String.format(contentFormat, oldName), 259200000L, (List) null, rewardItems, true, false);
    }

    public int getCharacterStageCount(Player player) {
        HashMap<Integer, Character> charaMap = player.getCharacters().getCharMap();
        int count = 0;
        Collection<Character> charas = charaMap.values();
        for (Character chara : charas) {
            count += chara.getStageLevel();
        }
        return count;
    }

    public int getPlayerStarCount(Player player) {
        HashMap<Integer, Character> charaMap = player.getCharacters().getCharMap();
        int count = 0;
        Collection<Character> charas = charaMap.values();
        for (Character chara : charas) {
            count += ManagerPool.getInstance().characterManager.getGrowthLv(chara);
        }
        return count;
    }
}
